@@ -1,5 +1,4 @@
-"""Utility code for PDR Toolbox.
-"""
+### Utility code for PDR Toolbox.
 
 import datetime
 import os.path
@@ -12,6 +11,7 @@ from copy import deepcopy
 import astropy.units as u
 from astropy.constants import k_B
 from astropy.table import Table
+from astropy.units.format.latex import Latex
 
 from pdrtpy import version
 
@@ -19,6 +19,11 @@ from pdrtpy import version
 _RFS_UNIT_ = u.erg/(u.second*u.cm*u.cm)
 _OBS_UNIT_ = u.erg/(u.second*u.cm*u.cm*u.sr)
 _CM = u.Unit("cm")
+_CM2 = u.Unit("cm-2")
+_K = u.Unit("K")
+_KKMS = u.Unit("K km s-1")
+LOGE = np.log10(np.e)
+LN10 = np.log(10)
 
 # ISRF in other units
 #The wavelength of 1110 Ang is the longest wavelength for H2 excitation,
@@ -28,11 +33,58 @@ _CM = u.Unit("cm")
 # See Weingartner and Draine 2001, ApJS, 134, 263, section 4.1
 
 habing_unit = u.def_unit('Habing',1.60E-3*_RFS_UNIT_)
+r"""The Habing radiation field unit 
+
+   :math:`{\rm 1~Habing = 1.6\times 10^{-3}~erg~s^{-1}~cm^{-2}}`
+"""
 u.add_enabled_units(habing_unit)
 draine_unit = u.def_unit('Draine',2.72E-3*_RFS_UNIT_)
+r"""The Draine radiation field unit
+
+    :math:`{\rm 1~Draine = 2.72\times10^{-3}~erg~s^{-1}~cm^{-2}}`
+"""
 u.add_enabled_units(draine_unit)
 mathis_unit = u.def_unit('Mathis',1.81E-3*_RFS_UNIT_)
+r"""The Mathis radiation field unit
+
+    :math:`{\rm 1~Mathis = 1.81\times10^{-3}~erg~s^{-1}~cm^{-2}}`
+"""
 u.add_enabled_units(mathis_unit)
+
+_rad_title = dict()
+_rad_title['Habing'] = '$G_0$'
+_rad_title['Draine'] = '$\chi$'
+_rad_title['Mathis'] = 'FUV'
+
+def _nbversion():
+    return open("../VERSION","r").readline().strip(*"\n")
+def check_nb():
+    nbv = _nbversion()
+    ver = version()
+    if nbv != ver:
+        # How do I suppress the stacktrace of the warning itself?!
+    #    warnings.warn("The version of this notebook does not match the version of pdrtpy. You may be missing some functionality.  For best results, upgrade both pdtpry and pdrtpy-nb to the latest version.",stacklevel=0)
+        print(f"WARNING: Your version of the PDRT notebooks does [{nbv:s}] not match your version of pdrtpy [{ver:s}]. You may be missing some functionality.  For best results, upgrade both pdtpry and pdrtpy-nb to the latest version. See https://pdrtpy.readthedocs.io")
+    else:
+        print("Your PDRT notebooks version matches your pdrtpy version -- hooray!")
+def get_rad(key):
+    """Get radiation field symbol (LaTeX) given radiation field unit.
+    If key is unrecognized, 'FUV Radiation Field' is returned.
+
+    :param key: input field unit name, e.g. 'Habing', 'Draine' or an :class:`astropy.units.Unit`
+    :returns: LaTeX string for the radiation field symbol e.g., :math:`G_0`, :math:`\chi`
+    :type key: str or :class:`astropy.units.Unit`
+    :rtype: str
+    """
+    skey = str(key) # in case the passed key was a Unit
+    if skey in _rad_title:
+        return _rad_title[skey]
+    else:
+        return "FUV"
+
+# this didn't work
+#density_unit = u.def_unit("1/cm3",1/(u.cm*u.cm*u.cm))
+#u.add_enabled_units(density_unit)
     
 #See https://stackoverflow.com/questions/880530/can-modules-have-properties-the-same-way-that-objects-can
 # only works python 3.8+??
@@ -90,20 +142,20 @@ def root_path():
     """
     return Path(__file__).parent
 
-def data_dir():
+def testdata_dir():
     """Project test data directory, including trailing slash
 
     :rtype: str
     """
     return os.path.join(root_dir(),'testdata/')
 
-def get_data(filename):
+def get_testdata(filename):
     """Get fully qualified pathname to FITS test data file.
 
     :param filename: input filename, no path
     :type filename: str
     """
-    return data_dir()+filename
+    return testdata_dir()+filename
 
 def model_dir():
     """Project model directory, including trailing slash
@@ -284,15 +336,20 @@ def to(unit,image):
 
      :param unit: identifying the unit to convert to 
      :type unit: string or `astropy.units.Unit` 
-     :param image: the image to convert. It must have a `numpy.ndarray`
-        data member and `astropy.units.Unit` unit member.
+     :param image: the image to convert. It must have a :class:`numpy.ndarray`
+        data member and :class:`astropy.units.Unit` unit member.
      :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
      :return: an image with converted values and units
   """
+  #print("converting [%s] in %s to [%s]"%(image.unit,image.header["TITLE"],unit))
+  #@todo check out NDDataArray.convert_unit_to
+  # https://docs.astropy.org/en/stable/api/astropy.nddata.NDDataArray.html#astropy.nddata.NDDataArray.convert_unit_to
+    # todo equivalencies needed for e.g. temperature
   value = image.unit.to(unit)
   newmap = deepcopy(image)
   newmap.data = newmap.data * value
   newmap.unit = u.Unit(unit)
+  #@TODO deal with identifier.
   # deal with uncertainty in Measurements.
   if getattr(newmap,"_uncertainty") is not None:
      newmap._uncertainty.array = newmap.uncertainty.array * value
@@ -306,8 +363,8 @@ def toHabing(image):
 
      between 6eV and 13.6eV (912-2066 :math:`\unicode{xC5}`).  See `Weingartner and Draine 2001, ApJS, 134, 263 <https://ui.adsabs.harvard.edu/abs/2001ApJS..134..263W/abstract>`_, section 4.1 
 
-     :param image: the image to convert. It must have a `numpy.ndarray`
-        data member and `astropy.units.Unit` unit member.
+     :param image: the image to convert. It must have a :class:`numpy.ndarray`
+        data member and :class:`astropy.units.Unit` unit member.
      :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
      :return: an image with converted values and units
   """
@@ -320,8 +377,8 @@ def toDraine(image):
 
      between 6eV and 13.6eV (912-2066 :math:`\unicode{xC5}`).  See `Weingartner and Draine 2001, ApJS, 134, 263 <https://ui.adsabs.harvard.edu/abs/2001ApJS..134..263W/abstract>`_, section 4.1 
 
-     :param image: the image to convert. It must have a `numpy.ndarray`
-        data member and `astropy.units.Unit` unit member.
+     :param image: the image to convert. It must have a :class:`numpy.ndarray`
+        data member and :class:`astropy.units.Unit` unit member.
      :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
      :return: an image with converted values and units
   """
@@ -334,8 +391,8 @@ def toMathis(image):
 
      between 6eV and 13.6eV (912-2066 :math:`\unicode{xC5}`).  See `Weingartner and Draine 2001, ApJS, 134, 263 <https://ui.adsabs.harvard.edu/abs/2001ApJS..134..263W/abstract>`_, section 4.1 
 
-     :param image: the image to convert. It must have a `numpy.ndarray`
-        data member and `astropy.units.Unit` unit member.
+     :param image: the image to convert. It must have a :class:`numpy.ndarray`
+        data member and :class:`astropy.units.Unit` unit member.
      :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
      :return: an image with converted values and units
   """
@@ -344,7 +401,7 @@ def toMathis(image):
 def tocgs(image):
   r"""Convert a radiation field strength image to :math:`{\rm erg~s^{-1}~cm^{-2}}`.
 
-     :param image: the image to convert. It must have a :class:`numpy.ndarray` data member and `astropy.units.Unit` unit member.
+     :param image: the image to convert. It must have a :class:`numpy.ndarray` data member and :class:`astropy.units.Unit` unit member.
      :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
      :return: an image with converted values and units
   """
@@ -376,7 +433,7 @@ def convert_integrated_intensity(image,wavelength=None):
 
   with :math:`\lambda`  in cm, the factor :math:`10^5` is to convert :math:`dV` in :math:`{\rm km~s}^{-1}` to :math:`{\rm cm~s}^{-1}`.
 
-  :param image: the image to convert. It must have a `numpy.ndarray` data member and `astropy.units.Unit` unit member. It's units must be K km/s
+  :param image: the image to convert. It must have a :class:`numpy.ndarray` data member and :class:`astropy.units.Unit` unit member or header BUNIT keyword. It's units must be K km/s
   :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
   :param wavelength: the wavelength of the observation. The default is to determine wavelength from the image header RESTFREQ keyword
   :type wavelength: :class:`astropy.units.Quantity`
@@ -388,7 +445,9 @@ def convert_integrated_intensity(image,wavelength=None):
   if f is not None and wavelength is None:
      # FITS restfreq's are in Hz
      wavelength = u.Quantity(f,"Hz").to(_CM,equivalencies=u.spectral())
-  if image.header.get("BUNIT",None) != "K km/s":
+  if image.header.get("BUNIT",None) is None:
+     raise Exception("Image BUNIT must be present and equal to 'K km/s'")
+  if u.Unit(image.header.get("BUNIT")) != _KKMS:
      raise Exception("Image BUNIT must be 'K km/s'")
   factor = 2E5*k_B/wavelength**3
   print("Converting K km/s to %s using Factor = %s"%(_OBS_UNIT_, "{0:+0.3E}".format(factor.decompose(u.cgs.bases))))
@@ -402,13 +461,32 @@ def convert_integrated_intensity(image,wavelength=None):
      newmap._uncertainty.unit = _OBS_UNIT_
   return newmap
 
+def convert_if_necessary(image):
+    r"""Helper method to convert integrated intensity units in an
+    image or Measurement from :math:`{\rm K~km~s}^{-1}` to :math:`{\rm
+    erg~s^{-1}~cm^{-2}~sr^{-1}}`. If a conversion is necessary, the
+    :meth:`convert_integrated_intensity` is called.  If not, the image
+    is returned unchanged.
+
+    :param image: the image to convert. It must have a :class:`numpy.ndarray` data member and :class:`astropy.units.Unit` unit member or a header BUNIT keyword. It's units must be :math:`{\rm K~km~s}^{-1}`. It must also have a header RESTFREQ keyword.
+    :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
+
+    :return: an image with converted values and units
+    """
+    _u1 = u.Unit(image.header["BUNIT"])
+    _u2 = u.Unit("K km s-1")
+    if _u1 == _u2:
+        return convert_integrated_intensity(image)
+    else:
+        return image
+
 def mask_union(arrays):
     """Return the union mask (logical OR) of the input masked arrays.
     This is useful when doing arithmetic on images that don't have identical masks
     and you want the most restrictive mask.
 
     :param arrays: masked arrays to unionize
-    :type arrays: numpy.ma.masked_array
+    :type arrays: :class:`numpy.ma.masked_array`
     :rtype: mask
     """
     z = list()
@@ -443,7 +521,7 @@ def has_single_axis(w):
 def squeeze(image):
   """Remove single-dimensional entries from image data and WCS.
 
-  :param image: the image to convert. It must have a `numpy.ndarray` data member and `astropy.units.Unit` unit member. 
+  :param image: the image to convert. It must have a :class:`numpy.ndarray` data member and :class:`astropy.units.Unit` unit member. 
   :type image: :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
  
   :return: an image with single axes removed
@@ -469,3 +547,55 @@ def squeeze(image):
     nax = "NAXIS"+str(i)
   
   return image
+
+def fliplabel(label):
+    """Given a label that has a numerator and a denominator separated by a '/', return the 
+    reciprocal label.  For example, if the input label is '(x+y)/z' return 'z/(x+y)'.  This 
+    method simply looks for the '/' and swaps the substrings before and after it.
+    
+    :param label: the label to flip
+    :type label: str
+    :return: the reciprocal label
+    :rtype: str
+    :raises ValueError: if the input label has no '/'
+    """
+    ii = label.index('/')
+    return label[ii+1:]+'/'+label[0:ii]
+
+# partly stolen from astropy.quanity.to_string, will also work with Measurements
+def float_formatter(quantity,precision):
+    format_spec = '.{}g'.format(precision)
+    number = Latex.format_exponential_notation(quantity.value, format_spec=format_spec)
+    # strip the $ signs
+    unit = quantity.unit.to_string('latex_inline')[1:-1]
+    return f'{number}~{unit}'
+
+
+def is_image(image):
+    """Check if a Measurement is an image. The be an image it must have a header with axes keywords and a WCS to be considered an image.  This is to distiguish Measurements that have a data array with more than one member from a true image. 
+    :param image: the image to check. It must have a :class:`numpy.ndarray` data member and :class:`astropy.units.Unit` unit member or a header BUNIT keyword.
+    :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
+    :return: True if it is an image, False otherwise.
+    """
+   
+    if getattr(image,"header",None) is None or getattr(image,"wcs",None) is None:
+        return False
+    if image.wcs.naxis is None or image.wcs.wcs is None:
+        return False
+    if image.wcs.naxis == 0: #naxis=1 ok -- a 1-D image is still an image.
+        return False
+    if image.wcs.wcs.ctype is None:
+        return False
+    return True
+
+def is_ratio(identifier):
+    # find() returns -1 if char not found.
+    # in our case, also rule out that the / is in the zeroth position.
+    return identifier.find('/') > 0
+
+def isEven(number):
+    return abs(number) % 2 == 0
+
+def isOdd(number):
+    return not isEven(number)
+
