@@ -1,7 +1,4 @@
-import itertools
-import collections
 from copy import deepcopy
-import datetime
 
 import numpy as np
 import numpy.ma as ma
@@ -11,8 +8,9 @@ from astropy.io import fits
 from astropy.io.fits.header import Header
 import astropy.wcs as wcs
 import astropy.units as u
-from astropy.table import Table
-from astropy.nddata import NDDataArray, CCDData, NDUncertainty, StdDevUncertainty, VarianceUncertainty, InverseVariance
+import astropy.stats as astats
+from astropy.table import Table, Column
+from astropy.nddata import NDDataArray, CCDData, StdDevUncertainty
 
 from ..tool.toolbase import ToolBase
 from ..plot.lineratioplot import LineRatioPlot
@@ -21,8 +19,21 @@ from ..modelset import ModelSet
 from ..measurement import Measurement
 
 class LineRatioFit(ToolBase):
-    """Tool to fit observations of flux ratios to a set of models"""
+    """LineRatioFit is a tool to fit observations of intensity ratios to a set of PDR models. It takes as input a set of observations with errors represented as :class:`~pdrtpy.measurement.Measurement` and  :class:`~pdrtpy.modelset.ModelSet` for the models to which the data will be fitted. The observations should be spectral line or continuum intensities.  They can be spatial maps or single pixel values. They should have the same spatial resolution.
+    
+The models to be fit are stored as intensity ratios. The input observations will be use to create ratios that correspond to models. From there a minimization fit is done to determine the density and radiation field that best fit the data.At least 3 observations are needed in order to make at least 2 ratios. With fewer ratios, no fitting can be done. More ratios generally means better determined density and radiation field, assuming the data are consistent with each other.
+
+Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view the results.
+    
+
+:param modelset: The set of PDR models to use for fitting.
+:type modelset: :class:`~pdrtpy.modelset.ModelSet`
+
+:param measurements: Input measurements to be fit.  
+:type measurements: list or dict of :class:`~pdrtpy.measurement.Measurement`. If dict, the keys should be the Measurement *identifiers*.  
+    """
     def __init__(self,modelset=ModelSet("wk2006",z=1),measurements=None):
+        super().__init__() # needed?
         if type(modelset) == str:
             # may need to disable this
             self._initialize_modelTable(modelset)
@@ -43,12 +54,14 @@ class LineRatioFit(ToolBase):
         self._reduced_chisq = None
         self._likelihood = None
         self.radiation_field_unit = None
+        self.radiation_field_type = None
         self.density_unit = None
-        self._plotter = LineRatioPlot(self)
+        self.density_type = None
+        #self._plotter = LineRatioPlot(self) #not needed
     
-    def _set_measurementnaxis(self):
-        fk = utils.firstkey(self._measurements)
-        self._measurementnaxis = len(self._measurements[fk].shape)
+    #def _set_measurementnaxis(self):
+    #    fk = utils.firstkey(self._measurements)
+    #    self._measurementnaxis = len(self._measurements[fk].shape)
 
     @property
     def modelset(self):
@@ -82,20 +95,29 @@ class LineRatioFit(ToolBase):
         return list(self._observedratios.keys())
 
 
-    @property
-    def has_maps(self):
-        '''Are the Measurements used map-based or pixel-based?.
+#    @property
+#    def has_maps(self):
+#        '''Are the Measurements used map-based?. (i.e., have 2 spatial axes)
+#        
+#        :returns: True, if the observational inputs are spatial maps, False otherwise
+# 
+#        :rtype: bool
+#        '''
+#        
+#        return self._measurementnaxis > 1
+#    @property
+#    
+#    def has_vectors(self):
+#        '''Are the Measurements used a Nx1 vector, e.g. read in from a table with #:meth:`~pdrtpy.Measurement.from_table`.
+#        
+#        :returns: True, if the observational inputs are a vector, False otherwise
+##        :rtype: bool
+#        '''
+#        return self._measurementnaxis == 1
         
-        :returns: True, if the observational inputs are spatial maps, False if they were single-pixel Measurements
- 
-        :rtype: bool
-        '''
-        return self._measurementnaxis > 1
-
     @property
     def ratiocount(self):
-        '''The number of ratios that match models available in the 
-           current :class:`~pdrtpy.modelset.ModelSet` given the current set of measurements
+        '''The number of ratios that match models available in the current :class:`~pdrtpy.modelset.ModelSet` given the current set of measurements
  
         :rtype: int
         '''
@@ -234,44 +256,28 @@ class LineRatioFit(ToolBase):
     
     def read_models(self,unit=u.dimensionless_unscaled):
         """Given a list of measurement IDs, find and open the FITS files that have matching ratios
-           and populate the _modelratios dictionary.  Uses :class:`pdrtpy.measurement.Measurement` as a 
-storage mechanism. 
+        and populate the _modelratios dictionary.  Uses :class:`pdrtpy.measurement.Measurement` as 
+        a storage mechanism. 
 
            :param  m: list of measurement IDS (string)
            :type m: list
            :param unit: units of the data 
            :type unit: string or astropy.Unit
         """
-        d = utils.model_dir()
-
-        self._modelratios = dict()
-        for (k,p) in self._modelset.find_files(self.measurementIDs):
-            thefile = d+p
-            self._modelratios[k] = Measurement.read(thefile,unit=unit)
-            # Here we assume the model naxis are self-consistent!
-            # If not, something much deeper is wrong.
-            self._modelnaxis = self._modelratios[k].wcs.naxis
-            # fix WK2006 models
-            if self._modelratios[k].wcs.wcs.cunit[0] == "":
-                self._modelratios[k].header["CUNIT1"] = "cm-3"
-                self._modelratios[k].wcs.wcs.cunit[0] = u.Unit("cm-3")
-#todo handle this in Measurement.read
-            else:
-                self._modelratios[k].header["CUNIT1"] = str(self._modelratios[k].wcs.wcs.cunit[0])
-            if self._modelratios[k].wcs.wcs.cunit[1] == "":
-                self._modelratios[k].header["CUNIT2"] = "Habing"
-                # Raises UnitScaleError:
-                # "The FITS unit format is not able to represent scales that are not powers of 10.  Multiply your data by 1.600000e-03."
-                # This causes all sorts of downstream problems
-                #self._modelratios[k].wcs.wcs.cunit[1] = utils.habing_unit
- 
-            else:
-                self._modelratios[k].header["CUNIT2"] = str(self._modelratios[k].wcs.wcs.cunit[1])
-            if not self.density_unit:
-                self.density_unit = self._modelratios[k].wcs.wcs.cunit[0]
-            if not self.radiation_field_unit:
+        self._modelratios = self._modelset.get_models(self.measurementIDs,model_type='ratio')
+        k = utils.firstkey(self._modelratios)
+        self._modelnaxis = self._modelratios[k].wcs.naxis
+        if not self.density_unit:
+            self.density_unit = self._modelratios[k].wcs.wcs.cunit[0]
+            self.density_type = self._modelratios[k].wcs.wcs.ctype[0]
+        if not self.radiation_field_unit:
+            self.radiation_field_unit = self._modelratios[k].wcs.wcs.cunit[1]
+            self.radiation_field_type = self._modelratios[k].wcs.wcs.ctype[1]
+            # for wk2006 models, Habing units in the Y axis cause problems when trying to assign them
+            # to a WCS (see note in modelset.py).  So get from header instead.
+            if self.radiation_field_unit.to_string() == '':
                 try:
-                    self.radiation_field_unit    = u.Unit(self._modelratios[k].header["CUNIT2"])
+                    self.radiation_field_unit = u.Unit(self._modelratios[k].header["CUNIT2"])
                 except KeyError:
                     raise Exception("Keyword CUNIT2 is required in file %s FITS header to describe units of interstellar radiation field"%thefile)
     
@@ -296,35 +302,100 @@ storage mechanism.
 
         # Check the coordinate systems only if there is more than one pixel
         m1 = self._measurements[utils.firstkey(self._measurements)]
-        if len(m1) != 0:
+        if utils.is_image(m1):
             if not self._check_header("CTYPE1"):
                raise Exception("CTYPE1 of your input Measurements do not match. Please ensure coordinates of all Measurements are the same.")
             if not self._check_header("CTYPE2"):
                raise Exception("CTYPE2 of your input Measurements do not match. Please ensure coordinates of all Measurements are the same.")
 
         #Only allow beam = None if single value measurements.
-        if len(m1) == 0 :
+        if not utils.is_image(m1):
             if self._check_header("BMAJ",None) or self._check_header("BMIN",None) or self._check_header("BPA",None):
                utils.warn(self,"No beam parameters in Measurement headers, assuming they are all equal!")
         #if not self._check_header("BUNIT") ...
 
-    def run(self):
-        '''Run the full computation using all the :class:`observations <pdrtpy.measurement.Measurement>` added.   This will check compatibility of input observations (e.g., beam parameters, coordinate types, axes lengths) and raise exceptions if the observations don't match each other.
+    def run(self,mask=None):
+        '''Run the full computation using all the :class:`observations <pdrtpy.measurement.Measurement>` added.   This will 
+        check compatibility of input observations (e.g., beam parameters, coordinate types, axes lengths) and 
+        raise exceptions if the observations don't match each other.
 
-           :raises Exception: if no models match the input observations.
+           :param mask: Indicate how to mask image observations (Measurements) before computing the density 
+                        and radiation field. Possible values are:
+
+             ['mad', multiplier]   - compute standard deviation using median absolute deviation (astropy.mad_std),
+                                     and mask out values between +/- multiplier*mad_std.  Example: ['mad',1.0]
+
+             ['data', (low,high)]  - mask based on data values, mask out data between low and high
+
+             ['clip', (low,high)]  - mask based on data values, mask out data below low and above high
+
+             ['error', (low,high)] - mask based on uncertainty plane, mask out data where the corresponding error pixel value 
+                                     is below low or above high
+
+                              None - Don't mask data. This is the default.
+
+           :type mask:  list or None
+
+           :raises Exception: if no models match the input observations, observations are not compatible, 
+                              or on unrecognized parameters
         '''
+        #@todo global masking for 'data', 'clip', 'error' not entirely useful unless all data/error have same ranges.
+        #need something like ['data',['key1':(low,hi), 'key2',(low,hi),...], which is very complicated.
+        # or data/error cut based on histogram 
         self._check_compatibility()
         self.read_models()
+        self._mask_measurements(mask)
         self._compute_valid_ratios()
         if self.ratiocount == 0 :
             raise Exception("No models were found that match your data. Check ModelSet.supported_ratios.")
         # eventually need to check that the maps overlap in real space.
         self._compute_delta_sq()
         self._compute_chisq()
-        #self._write_chisq()
         self.compute_density_radiation_field()
      
 
+    def _mask_measurements(self,mask):
+        ''' Set the mask on the measurements based on noise characteristics.  This is so that
+            we don't compute garbage n,G0 where observed ratios are noise divided by noise.
+            
+           :param mask: Indicate how to mask image observations before computing the density and radiation field. See run()>
+        '''
+        if mask is None: return
+        if self._measurementnaxis == 0:
+            utils.warn(self,"Ignoring 'mask' parameter for single pixel observations")
+            return
+        if mask[0] == 'mad':
+            for k,v in self._measurements.items():
+                sigcut = mask[1]*astats.mad_std(v.data,ignore_nan=True)
+                print("Masking %s data between [%.1e,%.1e]"%(k,-sigcut,sigcut))
+                masked_data = ma.masked_inside(v.data,-sigcut,sigcut,copy=True)
+                # CCDData/NDData do not use MaskArrays underneath but two nddata.arrays. Why??
+                # Make a copy so we are keeping references to data copies lying around.
+                v.mask = masked_data.mask.copy()
+        elif mask[0] == 'data':
+            for k,v in self._measurements.items():
+                masked_data=ma.masked_inside(v.data,mask[1][0],mask[1][1],copy=True)
+                v.mask = masked_data.mask.copy()
+                print("Masking %s data between [%.1e,%.1e]"%(k,mask[1][0],mask[1][1]))
+        elif mask[0] == 'clip':
+            for k,v in self._measurements.items():
+                masked_data=ma.masked_outside(v.data,mask[1][0],mask[1][1],copy=True)
+                v.mask = masked_data.mask.copy()
+                print("Masking %s data outside [%.1e,%.1e]"%(k,mask[1][0],mask[1][1]))
+        elif mask[0] == 'error':
+            for k,v in self._measurements.items():
+                # error is StdDevUncertainty so must use _array to get at raw values
+                indices = np.where((v.error <= mask[1][0]) | (v.error >= mask[1][1]))
+                if v.mask is not None:
+                    v.mask[indices] = True
+                else:
+                    v.mask = np.full(v.data.shape,False)
+                    v.mask[indices] = True
+                print("Masking %s data where error outside [%.1e,%.1e]"%(k,mask[1][0],mask[1][1]))
+        else:
+            raise ValueError("Unrecognized mask parameter %s. Valid values are 'mad','data','error'"%mask[0])
+            
+
     def _compute_valid_ratios(self):
         '''Compute the valid observed ratio maps for the available model data'''
         if not self._check_measurement_shapes():
@@ -337,8 +408,8 @@ storage mechanism.
         for p in z:
             label = p["numerator"]+"/"+p["denominator"]
             # deepcopy workaround for bug: https://github.com/astropy/astropy/issues/9006
-            num = self._convert_if_necessary(self._measurements[p["numerator"]])
-            denom = self._convert_if_necessary(self._measurements[p["denominator"]])
+            num = utils.convert_if_necessary(self._measurements[p["numerator"]])
+            denom = utils.convert_if_necessary(self._measurements[p["denominator"]])
             self._observedratios[label] = deepcopy(num/denom)
             self._observedratios[label].meta = deepcopy(num.header)
             #@TODO create a meaningful header for the ratio map
@@ -351,9 +422,8 @@ storage mechanism.
         if "CII_158" in m and "FIR" in m:
             if "OI_63" in m:
                 lab="OI_63+CII_158/FIR"
-                #print(l)
-                oi = self._convert_if_necessary(self._measurements["OI_63"])
-                cii = self._convert_if_necessary(self._measurements["CII_158"])
+                oi = utils.convert_if_necessary(self._measurements["OI_63"])
+                cii = utils.convert_if_necessary(self._measurements["CII_158"])
                 a = deepcopy(oi+cii)
                 b = deepcopy(self._measurements["FIR"])
                 self._observedratios[lab] = a/b
@@ -361,9 +431,8 @@ storage mechanism.
                 self._ratioHeader("OI_63+CII_158","FIR",lab)
             if "OI_145" in m:
                 lab="OI_145+CII_158/FIR"
-                #print(ll)
-                oi = self._convert_if_necessary(self._measurements["OI_145"])
-                cii = self._convert_if_necessary(self._measurements["CII_158"])
+                oi = utils.convert_if_necessary(self._measurements["OI_145"])
+                cii = utils.convert_if_necessary(self._measurements["CII_158"])
                 aa = deepcopy(oi+cii)
                 bb = deepcopy(self._measurements["FIR"])
                 self._observedratios[lab] = aa/bb
@@ -387,10 +456,10 @@ storage mechanism.
            :type f: float
         '''
         if not self._modelratios: # empty list or None
-            raise Exception("No model data ready.  You need to call read_fits")
+            raise Exception("No model data ready.  Was read_models() called?")
             
         if self.ratiocount < 2 :
-            raise Exception("Not enough ratios to compute deltasq.  Need 2, got %d"%self.ratiocount)
+            raise Exception("Not enough ratios.  You need to provide at least 3 observations that can be used to compute 2 ratios that are covered by the ModelSet. From your observations, only %d ratios can be computed."%self.ratiocount)
 
         if not self._check_ratio_shapes():
             raise Exception("Observed ratio maps have different dimensions")
@@ -401,7 +470,7 @@ storage mechanism.
             modelpix = np.reshape(self._modelratios[r],sz)
 
             residuals = list()
-            mf = ma.masked_invalid(self._observedratios[r].flux)
+            mf = ma.masked_invalid(self._observedratios[r].value)
             me = ma.masked_invalid(self._observedratios[r].error)  
             #frac_error = f*modelpix  # this is actually slower than looping over modelpix
             s2 = me**2
@@ -418,8 +487,6 @@ storage mechanism.
                 _q = ma.masked_invalid(_q)
                 residuals.append(_q)
             # result order is g0,n,y,x
-            #print("Shape mr1 ",self._modelratios[r].shape,type(self._modelratios[r].shape[0]))
-            #print("Shape or2 ",self._observedratios[r].shape)#,type(self._observedratios[r].shape[0]))
 
             # Catch the case of a single pixel
             if len(self._observedratios[r].shape) == 0:
@@ -482,8 +549,8 @@ storage mechanism.
         self._reduced_chisq.header =  Header(deepcopy(self._chisq.header))
         self._fixheader(self._chisq)
         self._fixheader(self._reduced_chisq)
-        utils.setkey("BUNIT","Chi-squared",self._chisq)
-        utils.setkey("BUNIT",("Reduced Chi-squared (DOF=%d)"%self._dof),self._reduced_chisq)
+        utils.comment("Chi-squared",self._chisq)
+        utils.comment(("Reduced Chi-squared (DOF=%d)"%self._dof),self._reduced_chisq)
         self._makehistory(self._chisq)
         self._makehistory(self._reduced_chisq)
         
@@ -507,15 +574,11 @@ storage mechanism.
         # get the likelihood maxima of each pixel along the g,n axes
         z=np.amax(self._likelihood,(0,1))
         gi,ni,yi,xi=np.where(self._likelihood==z)
-        #print(gi)
-        #print(len(gi),len(ni),len(xi),len(yi))
         spatial_idx = (yi,xi)
         model_idx   = np.transpose(np.array([ni,gi]))
         # qq[:,:2] takes the first two columns of qq
         # [:,[1,0]] swaps those columns
         # np.flip would also swap them.
-        #print(qq[:,:2][:,[1,0]])
-        #print(np.flip(qq[:,:2]))
         fk = utils.firstkey(self._modelratios)
         fk2 = utils.firstkey(self._observedratios)
         newshape = self._observedratios[fk2].shape
@@ -533,7 +596,7 @@ storage mechanism.
         #self._density_radiation_field_header() 
         
     def compute_density_radiation_field(self):
-        '''Compute the best-fit density n and radiation field spatial maps 
+        '''Compute the best-fit density and radiation field spatial maps 
            by searching for the minimum chi-squared at each spatial pixel.'''
         if self._chisq is None or self._reduced_chisq is None: return
         
@@ -542,6 +605,7 @@ storage mechanism.
         mshape = self._modelratios[fk].shape
         # Wolfire 2006 models have NAXIS=2, while 2020+ have NAXIS=3.
         # Deal with it.
+        # @see Measurement squeeze parameter. This should no longer be needed
         if self._modelnaxis == 2:
             firstindex = 0
             secondindex = 1
@@ -559,13 +623,6 @@ storage mechanism.
         gnxy = np.where(self._reduced_chisq==rchi_min)
         gi = gnxy[firstindex]
         ni = gnxy[secondindex]
-        #print("GI ",gi)
-        #print("NI ",ni)
-        #print("len(rchimin) shape(rchimin) ",len(rchi_min),rchi_min.shape)
-        #print("len(gi) len(ni) ",len(gi),len(ni))
-        #print("shape(gi) shape(ni) ",np.shape(gi),np.shape(ni))
-        #print("gnxy shape , len(gnxy) ",np.shape(gnxy),len(np.shape(gnxy)))
-        #print("chi_min shape, len(shape(chi_min)) ",np.shape(chi_min),len(np.shape(chi_min)))
         if len(gnxy) >= 4:
             # astronomical spatial indices
             spatial_idx = (gnxy[thirdindex],gnxy[fourthindex])
@@ -576,26 +633,20 @@ storage mechanism.
         if self._modelnaxis == 3:
             # add 3rd axis to model_idx
             model_idx = np.insert(model_idx,0,[0],axis=1)
-        #print("MODEL INDEX: ",model_idx)
-        # qq[:,:2] takes the first two columns of qq
-        # [:,[1,0]] swaps those columns
-        # np.flip would also swap them.
-        #print(qq[:,:2][:,[1,0]])
-        #print(np.flip(qq[:,:2]))
         fk2 = utils.firstkey(self._observedratios)
         newshape = self._observedratios[fk2].shape
         g0 =10**(self._modelratios[fk].wcs.wcs_pix2world(model_idx,0))[:,1]
         n =10**(self._modelratios[fk].wcs.wcs_pix2world(model_idx,0))[:,0]
-        #print("G ",g0)
-        #print("N ",n)
-
         self._radiation_field=deepcopy(self._observedratios[fk2])
-        if spatial_idx == 0:
+        if spatial_idx == 0 and len(newshape) == 0:
             self._radiation_field.data=g0[0]
             self._radiation_field.uncertainty.array=float("NAN")
         else:
-            # note this will reshape g0 in radiation_field for us!
-            self._radiation_field.data[spatial_idx]=g0
+            if len(newshape) == 1: # Measurement with data vector
+                self._radiation_field.data = g0
+            else: #Measurement with image
+                # note this will reshape g0 in radiation_field for us! 
+                self._radiation_field.data[spatial_idx]=g0
             # We cannot mask nans because numpy does not support writing
             # MaskedArrays to a file. Will get a not implemented error.
             # Therefore just copy the nans over from the input observations.
@@ -608,11 +659,15 @@ storage mechanism.
         self._radiation_field.uncertainty.unit = self.radiation_field_unit
 
         self._density=deepcopy(self._observedratios[fk2])
-        if spatial_idx == 0:
+        if spatial_idx == 0 and len(newshape) == 0:
             self._density.data=n[0]
             self._density.uncertainty.array=float("NAN")
         else:
-            self._density.data[spatial_idx]=n
+            if len(newshape) == 1: # Measurement with data vector
+                self._density.data = n
+            else: #Measurement with image
+                # note this will reshape g0 in radiation_field for us! 
+                self._density.data[spatial_idx]=n
             self._density.data[np.isnan(self._observedratios[fk2])] = np.nan
             # kluge because we dont know how to properly calcultate undertainty on this.
             #self._density.uncertainty.array=np.zeroes(self._density.uncertainty.array)
@@ -628,7 +683,6 @@ storage mechanism.
 
         # now save copies of the 2D min chisquares
         self._chisq_min=deepcopy(self._observedratios[fk2])
-        #print("chisq_min shape, len(chisq_min) ",np.shape(self._chisq_min),len(np.shape(self._chisq_min)))
         if spatial_idx == 0:
             self._chisq_min.data = chi_min
         else:
@@ -695,15 +749,14 @@ storage mechanism.
         :param image: The image to which to add the header values
         :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
         '''
-        # @TODO make these headers compliant with inputs (e.g. requested units)
         if self._modelnaxis == 2:
             naxis = len(image.shape)
         else:
             naxis = len(image.shape)-1
         ax1=str(naxis-1)
         ax2=str(naxis)
-        utils.setkey("CTYPE"+ax1,"Log(Volume Density)",image)
-        utils.setkey("CTYPE"+ax2,"Log(Radiation Field)",image)
+        utils.setkey("CTYPE"+ax1,self.density_type,image)
+        utils.setkey("CTYPE"+ax2,self.radiation_field_type,image)
         utils.setkey("CUNIT"+ax1,str(self.density_unit),image)
         utils.setkey("CUNIT"+ax2,str(self.radiation_field_unit),image)
 
@@ -733,16 +786,40 @@ storage mechanism.
         # convert from OrderedDict to astropy.io.fits.header.Header
         self._density.header = Header(self._density.header)
         self._radiation_field.header = Header(self._radiation_field.header)
+        self._density._identifier = "H2 Volume Density"
+        self._radiation_field._identifier = "Radiation Field"
+        
+    @property
+    def table(self):
+        #@TODO: make this work for map data ?
+        '''Construct the table of input Measurements, and if the fit has been run, the density, radiation field, and :math:`\chi^2` values
+        
+        :rtype: :class:`astropy.table.Table`
+        '''
+        v = self._measurements.values()
+        # This only works for astropy version >= 4.1
+        # and for some reason requirements.txt did not install it
+        # for a test user.
+        #t = Table(self._measurements,
+        #          units=[m.unit for m in v]
+        #          )
+        t = Table()
+        cols = [Column(data=d,unit=d.unit) for d in v]
+        t.add_columns(cols=cols, names=[m.id for m in v])
+        if self._observedratios is not None:
+            v = self._observedratios.values()
+            cols = [Column(data=d,unit=d.unit) for d in v]
+            t.add_columns(cols=cols, names=[m.id for m in v])
 
-       
-    def _convert_if_necessary(self,image):
-        """If the input image has units of K km/s convert it to intensity
+        if self.radiation_field is not None:
+            t.add_column(col=Column(self.radiation_field, unit=self.radiation_field_unit), name=self.radiation_field.id)
 
-        :param image: The image to which to add the header values
-        :type image: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
-        """
-        if image.header["BUNIT"] == "K km/s":
-            return utils.convert_integrated_intensity(image)
-        else:
-            return image
+        if self.density is not None:
+            t.add_column(col=Column(self.density, unit=self.density_unit), name=self.density.id)
 
+        if self._chisq_min is not None:
+            t.add_column(col=Column(self._chisq_min, unit=None), name="Chi-square")
+
+        for j in t.columns:
+            t[j].format = '3.2E'
+        return t
